import { NextFunction, Request, Response } from "express";
import { Post } from "../entity/Post";
import path from "path";
import fs from 'fs';
import crypto from 'crypto';
import jwt, { JwtPayload } from "jsonwebtoken";
import { AppDataSource } from "../data-source";

// Updates or creates a post and handles things like deleting old post's media
const handlePostData = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
    const token = req.cookies.jwt;
    const user = (jwt.decode(token) as JwtPayload)

    const post = new Post();

    const { message } = req.body;
    if (req.method == "PUT") {
        // Delete old post data if it was media
        const postToUpdate = (await AppDataSource.manager.findOneBy(Post, { id: Number.parseInt(req.params.postId) }));

        if (postToUpdate.type == 1) {
            const filename = postToUpdate.message;
            fs.unlinkSync(`${process.env.UPLOAD_DESTINATION}/${filename}`);
        }
    }
    if (req.file) {
        const extension = path.extname(req.file.originalname).toLowerCase()
        if ([".png", ".jpg", ".jpeg", ".webp", ".mp4", ".webm"].indexOf(extension) < 0) {
            res.status(400).send("Unknown mime type");
            return;
        }
        const buffer = fs.readFileSync(req.file.path);
        const hash = crypto.createHash('md5');
        hash.update(buffer);
        const newFilename = `${hash.digest('hex')}${extension}`;
        fs.copyFileSync(`./${req.file.path}`, `${process.env.UPLOAD_DESTINATION}/${newFilename}`);
        fs.unlinkSync(`./${req.file.path}`);
        post.message = newFilename;
        post.type = 1;
    } else {
        post.type = 0;
        post.message = message;
    }
    if (req.method == "POST") post.date = new Date().toISOString();
    post.authorId = user.id;
    res.locals.post = post;
    next();
}

export default { handlePostData };